# -*- coding: utf-8 -*-
# @FileName  :file_util.py
# @Time      :2023/1/2 15:57
# @Author    :yaoys
# @Desc      :文件工具类，读取文件，写入文件等
import os
from typing import List, Optional, Union

import pandas as pd


# 使用pandas读取csv文件
def read_csv(file_path: Union[str] = None, encoding='UTF-8', need2_dataframe=False):
    if file_path is None:
        raise Exception('the file is not exist')

    data = pd.read_csv(filepath_or_buffer=file_path, encoding=encoding)
    if need2_dataframe:
        data = pd.DataFrame(data=data)
    return data


# 写入csv文件
def to_csv(data: Union[pd.DataFrame] = None, file_path=None, encoding='UTF-8', index=False, header: Union[bool, List[str]] = True, sep=','):
    '''

    :param data: 要写入的数据,格式为使用pandas读取的数据
    :param file_path: 文件路径
    :param encoding: 编码格式
    :param index: 是否写入索引，默认是FALSE
    :param header: 是否保留列名，默认为TRUE，为TRUE则使用pandas读取的数据的列名不做其他修改，如果为list，则设置新列名
    :param sep: 分隔字符
    :return:
    '''
    if file_path is None:
        raise Exception('if save file,the file_path is required')

    if data is None:
        raise Exception('if save file,the data is required')

    data.to_csv(path_or_buf=file_path, index=index, encoding=encoding, sep=sep, header=header)


# 读取TXT文件，读取模式为按行读取所有，读取某一行,默认为按行读取所有，返回为一个list,如果读取某一行，需要设置line_count参数
def read_txt(file_path: Union[str] = None, encoding='UTF-8', read_model: Union[str] = 'all', line_count: Union[int] = -1):
    '''
    :param file_path:
    :param encoding:
    :param read_model: all(read all) or read_line(read one line)
    :param line_count: 读取的行数
    :return:
    '''
    result = []

    if os.path.exists(file_path) is False:
        raise Exception('The file path is error')

    if read_model == 'all':
        with open(file_path, 'r', encoding=encoding) as f:
            for line in f.readlines():
                # 去除文本中的换行符
                line = line.strip('\n')
                result.append(line)
    elif read_model == 'read_line':
        if line_count < 0:
            raise Exception('If you want to read line, you must set line_count >=0')

        count = 0

        with open(file_path, 'r', encoding=encoding) as f:
            if line_count > len(f.readlines()):
                raise Exception('The number of rows read is greater than the total number of rows')

        with open(file_path, 'r', encoding=encoding) as f:
            for line in f.readlines():
                if line_count == count:
                    # 去除文本中的换行符
                    line = line.strip('\n')
                    result.append(line)
                    break
                count = count + 1

    return result


def to_txt(data: Optional[List] = None, file_path: Union[str] = None, encoding='UTF-8'):
    '''

    :param data: 要写入的数据，格式为list
    :param file_path: 文件路径
    :param encoding: 编码
    :return:
    '''
    if data is None:
        raise Exception('The data is None')

    if isinstance(data, list) is False:
        raise Exception('The data must is list')

    if file_path is None:
        raise Exception('The file path is error')
    f = open(file_path, 'w', encoding=encoding)
    # into txt
    for line in data:
        f.writelines(str(line) + '\n')
    f.close()



# -*- coding: utf-8 -*-
# @FileName  :split_files.py
# @Time      :2022/10/24 21:39
# @Author    :yaoys
# @Desc      : 数据集按照一定的比例划分训练集和测试集
import csv
import os

from sklearn.model_selection import train_test_split


class SplitFiles(object):
    def __init__(self, **kwargs):
        """初始化"""
        # 要划分的数据集路径
        self.file_path = kwargs.get('file_path', None)
        # 训练数据集的比例
        self.train_size = kwargs.get('train_size', 0.8)
        # 测试数据集的比例
        self.test_size = kwargs.get('test_size', 1 - self.train_size)
        # 随机种子
        self.random_state = kwargs.get('random_state', 42)
        # 数据集数组
        self.data_list = []
        # 训练数据集数组
        self.train_data_list = []
        # 测试数据集数组
        self.test_data_list = []
        self.is_train = False
        self.is_test = False

        # 训练数据集保存地址
        self.train_data_path = kwargs.get('train_data_path', None)
        # 测试数据集保存地址
        self.test_data_path = kwargs.get('test_data_path', None)

        # self.logger = getLogger(log_name='SplitFiles', log_level=MY_LOG_INFO, save_log2_file=False)

        if self.file_path is None:
            raise Exception('The file_path is None')
        if self.train_data_path is None:
            raise Exception('The train_data_path is None')
        if self.test_data_path is None:
            raise Exception('The test_data_path is None')
        if self.train_size + self.test_size != 1:
            raise Exception('The train_size and test_size add must is 1')

    # 读取文件，并将文件的每一行保存在数组中
    def read_file(self):
        if os.path.exists(self.file_path) is False:
            raise Exception('The file path is error')

        with open(self.file_path, "r", encoding='utf-8') as data_file:
            for line in data_file:
                self.data_list.append(line.strip())

    # 按照比例划分数据
    def splitFile(self):

        # 文件格式只能是csv或者TXT
        if os.path.splitext(self.train_data_path)[1] not in ['.csv'] and os.path.splitext(self.test_data_path)[1] not in ['.csv'] \
                and os.path.splitext(self.train_data_path)[1] not in ['.txt'] and os.path.splitext(self.test_data_path)[1] not in ['.txt']:
            raise Exception('The file type must csv or txt')
        # log_info('load data.....', my_logger=self.logger)
        # 读取所有的数据
        self.read_file()
        # log_info('split data', my_logger=self.logger)
        # 使用sklearn划分训练数据集和测试数据集
        self.train_data_list, self.test_data_list = train_test_split(self.data_list, test_size=self.test_size, train_size=self.train_size, random_state=self.random_state)
        # 保存文件
        # log_info('save train data and test data.....', my_logger=self.logger)
        self.save_result()

    # 保存划分好的数据集
    def save_result(self):
        try:
            # 该方法返回两个元素, 第一个是路径去掉后缀的部分, 第二个是文件后缀
            # 如果是Excel文件
            if os.path.splitext(self.train_data_path)[1] in ['.csv']:
                self.is_train = True
                self.is_test = False
                self.save_excel_result()
            if os.path.splitext(self.test_data_path)[1] in ['.csv']:
                self.is_train = False
                self.is_test = True
                self.save_excel_result()
            #     txt文件
            if os.path.splitext(self.train_data_path)[1] in ['.txt']:
                self.is_train = True
                self.is_test = False
                self.save_txt_result()
            if os.path.splitext(self.test_data_path)[1] in ['.txt']:
                self.is_train = False
                self.is_test = True
                self.save_txt_result()

        except:
            print('error')

    def save_txt_result(self):
        train_file = None
        test_file = None
        try:
            if self.is_train is True and self.is_test is False:
                self.exists_and_remove_file()
                # log_info('save train txt data.....', my_logger=self.logger)
                train_file = open(self.train_data_path, 'a', encoding='utf-8')

                for i in range(0, len(self.train_data_list)):
                    train_file.write(self.train_data_list[i] + '\n')

                train_file.close()

            if self.is_train is False and self.is_test is True:
                self.exists_and_remove_file()
                # log_info('save train txt data.....', my_logger=self.logger)
                test_file = open(self.test_data_path, 'a', encoding='utf-8')

                for i in range(0, len(self.test_data_list)):
                    test_file.write(self.test_data_list[i] + '\n')

                test_file.close()

        except:
            if train_file is not None:
                train_file.close()
            if test_file is not None:
                test_file.close()
            print('')

        self.is_train = False
        self.is_test = False

    def save_excel_result(self):
        if self.is_train is True and self.is_test is False:
            self.exists_and_remove_file()
            # log_info('save train excel data.....', my_logger=self.logger)
            f = open(self.train_data_path, 'a', encoding='utf-8', newline='')
            # 2. 基于文件对象构建 csv写入对象
            csv_writer = csv.writer(f)

            for i in range(0, len(self.train_data_list)):
                data_row = self.train_data_list[i]
                row = []
                for j in range(0, len(data_row.split(','))):
                    row.append(data_row.split(',')[j])
                csv_writer.writerow(row)

            f.close()
        if self.is_train is False and self.is_test is True:
            self.exists_and_remove_file()
            # log_info('save test excel data.....', my_logger=self.logger)
            f = open(self.test_data_path, 'a', encoding='utf-8', newline='')
            # 2. 基于文件对象构建 csv写入对象
            csv_writer = csv.writer(f)

            for i in range(0, len(self.test_data_list)):
                data_row = self.test_data_list[i]
                row = []
                for j in range(0, len(data_row.split(','))):
                    row.append(data_row.split(',')[j])
                csv_writer.writerow(row)
            f.close()
        self.is_train = False
        self.is_test = False

    def exists_and_remove_file(self):
        if self.is_train is True and self.is_test is False:
            if os.path.exists(self.train_data_path):
                # log_info('exists train file and remove the file', my_logger=self.logger)
                os.remove(self.train_data_path)

        if self.is_train is False and self.is_test is True:
            if os.path.exists(self.test_data_path):
                # log_info('exists test file and remove the file', my_logger=self.logger)
                os.remove(self.test_data_path)



# -*- coding: UTF-8 -*-

from yaoysTools.log.logutil import MY_LOG_INFO, getLogger
from yaoysTools.progress.bar import Bar, ChargingBar, FillingCirclesBar, FillingSquaresBar, IncrementalBar, PixelBar, ShadyBar
from yaoysTools.progress.spinner import LineSpinner, MoonSpinner, PieSpinner, PixelSpinner, Spinner

PROGRESS_TYPE_BAR = 'Bar'
PROGRESS_TYPE_CHARGING_BAR = 'ChargingBar'
PROGRESS_TYPE_FILLINGSQUARESBAR = 'FillingSquaresBar'
PROGRESS_TYPE_FILLINGCIRCLESBAR = 'FillingCirclesBar'
PROGRESS_TYPE_INCREMENTALBAR = 'IncrementalBar'
PROGRESS_TYPE_PIXELBAR = 'PixelBar'
PROGRESS_TYPE_SHADYBAR = 'ShadyBar'
PROGRESS_TYPE_SPINNER = 'Spinner'
PROGRESS_TYPE_PIESPINNER = 'PieSpinner'
PROGRESS_TYPE_MOONSPINNER = 'MoonSpinner'
PROGRESS_TYPE_LINESPINNER = 'LineSpinner'
PROGRESS_TYPE_PIXELSPINNER = 'PixelSpinner'

# The show time type
# elapsed	elapsed time in seconds such as: 10s
# elapsed_td	elapsed as a timedelta (useful for printing as a string) such as: 0:00:15
# eta	avg * remaining such as 10s,9s,8s.....
# eta_td	eta as a timedelta (useful for printing as a string) such as 0:00:15,0:00:14,0:00:13...

PROGRESS_ELAPSED = 'elapsed'
PROGRESS_ELAPSED_TD = 'elapsed_td'
PROGRESS_ETA = 'eta'
PROGRESS_ETA_TD = 'eta_td'


class definitionProgress(object):
    def __init__(self, **kwargs):
        self.progress_type = kwargs.get('progress_type', PROGRESS_TYPE_BAR)
        self.count = kwargs.get('count', 0)
        self.message = kwargs.get('message', 'Progress')
        self.check_tty = kwargs.get('check_tty', False)
        self.is_save_log = kwargs.get('is_save_log', False)
        self.logger = kwargs.get('logger', None)
        self.log_path = kwargs.get('log_path', None)
        self.__progress_color = kwargs.get('progress_color', None)
        self.__need_show_time = kwargs.get('need_show_time', True)
        self.__show_time_type = kwargs.get('show_time_type', PROGRESS_ELAPSED_TD)

        if self.progress_type in (PROGRESS_TYPE_SPINNER, PROGRESS_TYPE_MOONSPINNER, PROGRESS_TYPE_PIESPINNER, PROGRESS_TYPE_LINESPINNER, PROGRESS_TYPE_PIESPINNER) \
                and self.__show_time_type in (PROGRESS_ETA, PROGRESS_ETA_TD):
            raise Exception("if you use spinner,you don't use eta and eta_td")
        if self.is_save_log is True and self.logger is None:
            if self.log_path is None:
                raise Exception('The log_path is not None,if you want to save the progress to log')
            self.logger = getLogger(self.message, log_level=MY_LOG_INFO, log_path=self.log_path, is_only_file=True)

    def get_progress(self):
        if self.progress_type == PROGRESS_TYPE_BAR:
            return Bar(self.message, max=self.count, check_tty=self.check_tty, is_save_log=self.is_save_log, logger=self.logger, color=self.__progress_color, need_show_time=self.__need_show_time, show_time_type=self.__show_time_type)
        if self.progress_type == PROGRESS_TYPE_CHARGING_BAR:
            return ChargingBar(self.message, max=self.count, check_tty=self.check_tty, is_save_log=self.is_save_log, logger=self.logger, color=self.__progress_color, need_show_time=self.__need_show_time, show_time_type=self.__show_time_type)
        if self.progress_type == PROGRESS_TYPE_FILLINGSQUARESBAR:
            return FillingSquaresBar(self.message, max=self.count, check_tty=self.check_tty, is_save_log=self.is_save_log, logger=self.logger, color=self.__progress_color, need_show_time=self.__need_show_time, show_time_type=self.__show_time_type)
        if self.progress_type == PROGRESS_TYPE_FILLINGCIRCLESBAR:
            return FillingCirclesBar(self.message, max=self.count, check_tty=self.check_tty, is_save_log=self.is_save_log, logger=self.logger, color=self.__progress_color, need_show_time=self.__need_show_time, show_time_type=self.__show_time_type)
        if self.progress_type == PROGRESS_TYPE_INCREMENTALBAR:
            return IncrementalBar(self.message, max=self.count, check_tty=self.check_tty, is_save_log=self.is_save_log, logger=self.logger, color=self.__progress_color, need_show_time=self.__need_show_time, show_time_type=self.__show_time_type)
        if self.progress_type == PROGRESS_TYPE_PIXELBAR:
            return PixelBar(self.message, max=self.count, check_tty=self.check_tty, is_save_log=self.is_save_log, logger=self.logger, color=self.__progress_color, need_show_time=self.__need_show_time, show_time_type=self.__show_time_type)
        if self.progress_type == PROGRESS_TYPE_SHADYBAR:
            return ShadyBar(self.message, max=self.count, check_tty=self.check_tty, is_save_log=self.is_save_log, logger=self.logger, color=self.__progress_color, need_show_time=self.__need_show_time, show_time_type=self.__show_time_type)

        if self.progress_type == PROGRESS_TYPE_SPINNER:
            return Spinner(self.message, check_tty=self.check_tty, is_save_log=False, need_show_time=self.__need_show_time, show_time_type=self.__show_time_type)
        if self.progress_type == PROGRESS_TYPE_PIESPINNER:
            return PieSpinner(self.message, check_tty=self.check_tty, is_save_log=False, need_show_time=self.__need_show_time, show_time_type=self.__show_time_type)
        if self.progress_type == PROGRESS_TYPE_MOONSPINNER:
            return MoonSpinner(self.message, check_tty=self.check_tty, is_save_log=False, need_show_time=self.__need_show_time, show_time_type=self.__show_time_type)
        if self.progress_type == PROGRESS_TYPE_LINESPINNER:
            return LineSpinner(self.message, check_tty=self.check_tty, is_save_log=False, need_show_time=self.__need_show_time, show_time_type=self.__show_time_type)
        if self.progress_type == PROGRESS_TYPE_PIXELSPINNER:
            return PixelSpinner(self.message, check_tty=self.check_tty, is_save_log=False, need_show_time=self.__need_show_time, show_time_type=self.__show_time_type)



# _*_ coding: utf-8 _*_
import inspect
import logging
import os.path
import re
import time
import sys
import colorlog

MY_LOG_INFO = logging.INFO
MY_LOG_ERROR = logging.ERROR
MY_LOG_DEBUG = logging.DEBUG
MY_LOG_WARN = logging.WARNING
MY_LOG_ALL = logging.DEBUG


class mylog(object):

    def __init__(self, **kwargs):
        """
        :param
               log_name：日志名称
               log_path：日志路径，若save_log2_file=True，则该参数不能为空
               log_level：日志级别，如果file_log_level和stream_log_level为None，则默认使用log_level级别，如果log_level也为None，则三者同时默认级别为Info级别
               使用控制台会输出设置的级别及以上的日志，如设置info，会输出info级别和info级别以上的日志，这一点和日志文件的级别有本质的区别
               file_log_level：日志文件日志级别
               stream_log_level：控制台日志级别
               save_log2_file:是否需要保存至日志文件，默认是FALSE，如果为TRUE，则log_path不能为空
               is_only_file:日志只保存至日志文件，不在控制台输出，该参数主要用在进度条模块中，设置进度条保存在日志文件中时，需要设置该参数为True，保证不在控制台输出日志进度而保存到文件
        """

        self.__log_level = kwargs.get('log_level', MY_LOG_INFO)
        self.__file_log_level = kwargs.get('file_log_level', None)
        self.__stream_log_level = kwargs.get('stream_log_level', None)
        self.__save_log2_file = kwargs.get('save_log2_file', None)
        self.__log_name = kwargs.get('log_name', 'log')
        # 设置日志路径
        self.__log_path = kwargs.get('log_path', None)

        # 只需要保存到文件，不需要在控制台输出,默认是FALSE
        self.is_only_file = kwargs.get('is_only_file', None)

        # 不同级别的日志颜色
        # 以下转义码可用于格式字符串：
        #
        # {color}, fg_{color}, bg_{color}: 前景色和背景色。
        # bold, bold_{color}, fg_bold_{color}, bg_bold_{color}: 粗体/明亮的颜色。
        # thin, thin_{color}, fg_thin_{color}: 淡色（取决于终端）。
        # reset：清除所有格式（前景色和背景色）。
        # 可用的颜色名称为black、red、green、yellow、blue、 purple和。cyanwhite
        # 如bg_white,白色背景
        self.__log_colors_config = kwargs.get('log_colors_config', {
            'DEBUG': 'white',  # cyan white
            'INFO': 'green',
            'WARNING': 'yellow',
            'ERROR': 'red',
            'CRITICAL': 'bold_red,fg_bold_bold_red',
        })

        if self.__save_log2_file is True or self.is_only_file is True:
            if self.__log_path is None:
                raise Exception('The save_log2_file or is_only_file is true,you must set a log path')

        # 日志一共分成5个等级，从高到低分别是：CRITICAL = FATAL > ERROR > WARNING =  WARN > INFO > DEBUG > NOTSET = 0
        # 保存日志时,会按照日志等级，保存该级别日志以及以下日志
        if self.__log_level is None:
            self.__log_level = MY_LOG_DEBUG
        if self.__file_log_level is None:
            self.__file_log_level = self.__log_level
        if self.__stream_log_level is None:
            self.__stream_log_level = self.__log_level

        # 创建一个logger
        self.__logger = logging.getLogger(self.__log_name)
        self.__logger.setLevel(self.__log_level)

        # 设置一些变量
        self.__all_log_name = None
        self.__info_log_name = None
        self.__error_log_name = None
        self.__debug_log_name = None
        self.__warn_log_name = None

        self.__all_file_handler = None
        self.__info_file_handler = None
        self.__error_file_handler = None
        self.__debug_file_handler = None
        self.__warn_file_handler = None
        self.__stream_handler = None

        # 设置formatter
        # __format_str = 'time:%(asctime)s -log_name:%(name)s -level:%(levelname)-s -file_name:%(filename)-8s -fun_name:%(funcName)s -chain:%(chain)s - %(lineno)d line -message: %(message)s'
        # -chain:%(chain)s:调用链路
        # - %(lineno)d line：行数,这种方法获取的行数为本文件中的写入日志的方法，并不是调用写日志的代码的行数，所以不用这种方法
        # self.__format_str = 'log_time:%(asctime)s -log_name:%(name)s -log_level:%(levelname)-s -log_filename:%(log_filename)s -func_name:%(func_name)s -line_number:%(line_number)d line -message: %(message)s'
        self.__format_str = kwargs.get('format_str', '[%(asctime)s][%(name)s][%(levelname)s][%(log_filename)s][%(line_number)d line] message:%(message)s')

        self.__formatter = logging.Formatter(self.__format_str)
        # 控制台日志输出格式，按照不同的颜色
        self.__console_formatter = colorlog.ColoredFormatter(
            # 格式
            fmt='%(log_color)s' + self.__format_str,
            # datefmt='%Y-%m-%d  %H:%M:%S',
            # 颜色
            log_colors=self.__log_colors_config,
            datefmt='%Y-%m-%d %H:%M:%S'
        )

        # 创建日志日期
        self.__log_day = str(time.strftime('%Y-%m-%d', time.localtime(time.time())))

    def __set_stream_handler(self):
        # 再创建一个handler，用于输出到控制台
        self.__stream_handler = logging.StreamHandler()
        self.__stream_handler.setLevel(self.__stream_log_level)
        # 控制台日志格式
        self.__stream_handler.setFormatter(self.__console_formatter)
        # 控制台handler
        if not (self.__stream_handler in self.__logger.handlers):
            self.__logger.addHandler(self.__stream_handler)

    def __remove_handler(self):
        pass

    def __set_log_file_name(self):
        # 如果不存在已经设置日志路径
        if self.__log_path is None:
            # os.getcwd()获取当前文件的路径，此时日志保存在当前工具类下log目录下，不建议不指定日志文件目录
            path_dir = os.path.dirname(__file__) + '/log' + '/' + self.__log_day
            # 如果该项目下没有log目录，创建log目录
            if not os.path.exists(path_dir):
                os.makedirs(path_dir)
            # os.path.dirname()获取指定文件路径的上级路径
            log_path = os.path.abspath(os.path.dirname(path_dir)) + '/log'
        else:
            # 否则，设置了路径就使用用户设置的路径
            path_dir = self.__log_path + '/' + self.__log_day
            # 最后为目录，不存在则创建
            if not os.path.exists(path_dir):
                os.makedirs(path_dir)

        # 创建日志名称。
        # rq = time.strftime('%Y-%m-%d', time.localtime(time.time()))
        # 拼接日志文件路径名称
        self.__all_log_name = os.path.join(path_dir, self.__log_day + '-' + 'ALL' + '.log')
        self.__info_log_name = os.path.join(path_dir, self.__log_day + '-' + str(logging.getLevelName(MY_LOG_INFO)) + '.log')
        self.__error_log_name = os.path.join(path_dir, self.__log_day + '-' + str(logging.getLevelName(MY_LOG_ERROR)) + '.log')
        self.__debug_log_name = os.path.join(path_dir, self.__log_day + '-' + str(logging.getLevelName(MY_LOG_DEBUG)) + '.log')
        self.__warn_log_name = os.path.join(path_dir, self.__log_day + '-' + str(logging.getLevelName(MY_LOG_WARN)) + '.log')

    def __set_file_handler(self):
        # CRITICAL = FATAL > ERROR > WARNING =  WARN > INFO > DEBUG > NOTSET = 0
        # 创建一个通用的handler，用于写入日志文件，写入所有的日志级别
        self.__all_file_handler = logging.FileHandler(self.__all_log_name, encoding='utf-8')
        self.__all_file_handler.setLevel(self.__file_log_level)
        # 创建一个info_handler，用于写入INFO日志文件，只写入info级别及以下的日志
        self.__info_file_handler = logging.FileHandler(self.__info_log_name, encoding='utf-8')
        self.__info_file_handler.setLevel(MY_LOG_INFO)
        # 创建一个error_handler，用于写入ERROR日志文件，只写入error级别及以下的日志
        self.__error_file_handler = logging.FileHandler(self.__error_log_name, encoding='utf-8')
        self.__error_file_handler.setLevel(MY_LOG_ERROR)
        # 创建一个debug_handler，用于写入DEBUG日志文件，写入debug级别及以下的日志
        self.__debug_file_handler = logging.FileHandler(self.__debug_log_name, encoding='utf-8')
        self.__debug_file_handler.setLevel(MY_LOG_DEBUG)
        # 创建一个warn_handler，用于写入WARNING日志文件，写入WARNING级别及以下的日志
        self.__warn_file_handler = logging.FileHandler(self.__warn_log_name, encoding='utf-8')
        self.__warn_file_handler.setLevel(MY_LOG_WARN)

    def __set_file_formatter(self):
        # 日志文件日志格式
        self.__all_file_handler.setFormatter(self.__formatter)
        # 日志文件日志格式
        self.__info_file_handler.setFormatter(self.__formatter)
        # 日志文件日志格式
        self.__error_file_handler.setFormatter(self.__formatter)
        # 日志文件日志格式
        self.__debug_file_handler.setFormatter(self.__formatter)
        # 日志文件日志格式
        self.__warn_file_handler.setFormatter(self.__formatter)

    def __add_file_handler(self):
        # if not self.__logger.handlers:
        # 给logger添加handler
        # 如果保存到文件
        if not (self.__all_file_handler in self.__logger.handlers):
            # 所有日志级别handler，如果不配置，则无法写入文件
            self.__logger.addHandler(self.__all_file_handler)

        if not (self.__info_file_handler in self.__logger.handlers):
            # info日志级别handler，如果不配置，则无法写入文件
            self.__logger.addHandler(self.__info_file_handler)

        if not (self.__error_file_handler in self.__logger.handlers):
            # error日志级别handler，如果不配置，则无法写入文件
            self.__logger.addHandler(self.__error_file_handler)

        if not (self.__debug_file_handler in self.__logger.handlers):
            # debug日志级别handler，如果不配置，则无法写入文件
            self.__logger.addHandler(self.__debug_file_handler)

        if not (self.__warn_file_handler in self.__logger.handlers):
            # warning日志级别handler，如果不配置，则无法写入文件
            self.__logger.addHandler(self.__warn_file_handler)

    def get_logger(self):
        if self.__save_log2_file is True or self.is_only_file is True:
            self.__set_log_file_name()
            self.__set_file_handler()
            self.__set_file_formatter()
            self.__add_file_handler()

        # 如果设置不输出控制台为FALSE
        if self.is_only_file is False:
            self.__set_stream_handler()
        # 日志重复输出，此设置不确定是否有效
        self.__logger.propagate = False
        return self.__logger

    @staticmethod
    def __get_file_name_in_full_path(file_path):
        return file_path.split('/')[-1]

    # 以下方法来源于https://github.com/frankyaorenjie/Python-CLog
    # def get_meta_data(self):
    #     frames = inspect.stack()
    #     chain_list = []
    #     for i in range(0, len(frames)):
    #         _, file_path, _, func_name, _, _ = frames[i]
    #         file_name = self.__get_file_name_in_full_path(file_path)
    #         try:
    #             args = re.findall('\((.*)\)', frames[i + 1][-2][0])[0]
    #         except IndexError as e:
    #             func_result = self.__get_class_from_frame(frames[2][0])
    #             if func_result is None:
    #                 func_name = ''
    #                 args = ''
    #             else:
    #                 func_name = self.__get_class_from_frame(frames[2][0]).__name__
    #                 args = ''
    #         current_chain = '%s:%s(%s)' % (file_name, func_name, args)
    #         chain_list.append(current_chain)
    #     chain_list.reverse()
    #     return ' --> '.join(chain_list[-4:-2])

    @staticmethod
    def __get_class_from_frame(fr):
        args, _, _, value_dict = inspect.getargvalues(fr)
        if len(args) and args[0] == 'self':
            instance = value_dict.get('self', None)
            if instance:
                return getattr(instance, '__class__', None)
        return None


__self_my_log = None

__myLogger = None


def get_log(log_name='self_my_log', log_path=None, log_level=None, file_log_level=None,
            stream_log_level=None, save_log2_file=False, is_only_file=False):
    global __self_my_log
    __self_my_log = mylog(log_name=log_name, log_path=log_path, log_level=log_level, file_log_level=file_log_level,
                          stream_log_level=stream_log_level, save_log2_file=save_log2_file, is_only_file=is_only_file)
    return __self_my_log


def getLogger(log_name='self_my_log', log_path=None, log_level=None, file_log_level=None,
              stream_log_level=None, save_log2_file=False, is_only_file=False):
    global __self_my_log
    __self_my_log = get_log(log_name=log_name, log_path=log_path, log_level=log_level, file_log_level=file_log_level,
                            stream_log_level=stream_log_level, save_log2_file=save_log2_file, is_only_file=is_only_file)
    if __self_my_log is None:
        raise Exception('The global self_my_log is none,please set self_my_log')

    global __myLogger
    __myLogger = __self_my_log.get_logger()

    # 日志重复输出，此设置无效
    __myLogger.propagate = False
    return __myLogger


# def get_chain():
#     global __self_my_log
#     if __self_my_log is None:
#         return ''
#     else:
#         my_chain = mylog(log_name='self_my_log').get_meta_data()
#         return my_chain


def _get_file_name():
    # 获取调用该方法的文件的名称。0指的是当前文件，1表示上一个文件，以栈的方式保存调用链路
    file_name = sys._getframe(2).f_code.co_filename
    if file_name != '' and file_name.count('/') >= 3:
        return '/'.join(str(i) for i in file_name.split('/')[-1:])
    elif file_name != '' and file_name.count('\\') >= 3:
        return '\\'.join(str(i) for i in file_name.split('\\')[-1:])
    else:
        return file_name


def _get_code_function_name():
    # 调用该函数的函数名字，如果没有被调用，则返回<module>
    function_name = sys._getframe(2).f_code.co_name
    if '<module>' in function_name:
        function_name = '非方法调用(Not function)'
    return function_name


def _get_code_line_number():
    # 调用该函数的函数名字，如果没有被调用，则返回<module>
    return sys._getframe(2).f_lineno


# 'chain': get_chain() 输出调用链路，无实际作用
def log_info(message, my_logger=None):
    if my_logger is None and __myLogger is not None:
        my_logger = __myLogger
    if my_logger is None and __myLogger is None:
        my_logger = get_log(log_level=MY_LOG_INFO).get_logger()

    if my_logger.level != MY_LOG_INFO:
        my_logger.setLevel(MY_LOG_INFO)
    my_logger.info(message, extra={'log_filename': _get_file_name(), 'func_name': _get_code_function_name(), 'line_number': _get_code_line_number()})
    # 暂时取消以下代码，因为如果不取消在demo中调用a方法，只能输出a中的日志，其他的方法中不输出，原因是因为，每次调用的时候handler被清空了，导致控制台handler没有了
    # my_logger.handlers = []


def log_error(message, my_logger=None):
    if my_logger is None and __myLogger is not None:
        my_logger = __myLogger
    if my_logger is None and __myLogger is None:
        my_logger = get_log(log_level=MY_LOG_ERROR).get_logger()

    if my_logger.level != MY_LOG_ERROR:
        my_logger.setLevel(MY_LOG_ERROR)
    my_logger.error(message, extra={'log_filename': _get_file_name(), 'func_name': _get_code_function_name(), 'line_number': _get_code_line_number()})
    # my_logger.handlers = []


def log_warn(message, my_logger=None):
    if my_logger is None and __myLogger is not None:
        my_logger = __myLogger
    if my_logger is None and __myLogger is None:
        my_logger = get_log(log_level=MY_LOG_WARN).get_logger()

    if my_logger.level != MY_LOG_WARN:
        my_logger.setLevel(MY_LOG_WARN)
    my_logger.warning(message, extra={'log_filename': _get_file_name(), 'func_name': _get_code_function_name(), 'line_number': _get_code_line_number()})
    # my_logger.handlers = []


def log_debug(message, my_logger=None):
    if my_logger is None and __myLogger is not None:
        my_logger = __myLogger
    if my_logger is None and __myLogger is None:
        my_logger = get_log(log_level=MY_LOG_DEBUG).get_logger()
    if my_logger.level != MY_LOG_DEBUG:
        my_logger.setLevel(MY_LOG_DEBUG)
    # '''chain': get_chain(),'''
    my_logger.debug(message, extra={'log_filename': _get_file_name(), 'func_name': _get_code_function_name(), 'line_number': _get_code_line_number()})
    # my_logger.handlers = []


def get_log_level(my_logger=None):
    if my_logger is None:
        return -1
    else:
        return my_logger.level



import numpy as np
import pandas
from sklearn.model_selection import train_test_split


# 自定义划分测试数据集与训练数据集
def split_train_test(data, test_ratio=0.2):
    """
    :param
    data: 数据
    test_ratio:测试数据集的比例，默认0.2，按照2-8原则划分
    """

    if not isinstance(data, pandas.DataFrame):
        raise Exception('please use pandas read_csv read the file')

    # 设置随机数种子，保证每次生成的结果都是一样的
    np.random.seed(42)
    # permutation随机生成0-len(data)随机序列
    shuffled_indices = np.random.permutation(len(data))
    # test_ratio为测试集所占的半分比
    test_set_size = int(len(data) * test_ratio)
    test_indices = shuffled_indices[:test_set_size]
    train_indices = shuffled_indices[test_set_size:]
    # iloc选择参数序列中所对应的行
    return data.iloc[train_indices], data.iloc[test_indices]


def split_train_test_dev(data, train_ratio=0.6, test_ratio=0.2, dev_ratio=0.2, random_state=42):
    '''
    划分数据集为训练数据集，测试数据集，验证数据集，默认为6:2:2
    '''
    if not isinstance(data, pandas.DataFrame):
        raise Exception('please use pandas read_csv read the file')
    if train_ratio + test_ratio + dev_ratio != 1:
        raise Exception('The ratio count must is 1')
    # 如果数据集长度大于万级别以上，就按照8:1:1划分
    if len(data) >= 10000:
        train_ratio = 0.8
        test_ratio = 0.1
        dev_ratio = 0.1
    # 设置随机数种子，保证每次生成的结果都是一样的
    np.random.seed(random_state)
    # permutation随机生成0-len(data)随机序列，打乱所有数据的顺序，得到的是随机排序的数组
    shuffled_indices = np.random.permutation(len(data))
    # train_ratio为训练集所占的百分比
    train_set_size = int(len(data) * train_ratio)
    # 从开始到训练数据集长度为训练数据集
    train_indices = shuffled_indices[:train_set_size]
    # 训练数据集到训练+测试为测试数据集
    test_indices = shuffled_indices[train_set_size:train_set_size + int(len(data) * test_ratio)]
    # 剩下的部分就是验证数据集
    dev_indices = shuffled_indices[train_set_size + int(len(data) * test_ratio):]
    # iloc选择参数序列中所对应的行
    return data.iloc[train_indices], data.iloc[test_indices], data.iloc[dev_indices]


# 使用sklearn库函数
def split_train_test_sklearn(data, test_ration=0.2, train_ration=0.8, random_state=42):
    if not isinstance(data, pandas.DataFrame):
        raise Exception('please use pandas read_csv read the file')
    return train_test_split(data, test_size=test_ration, train_size=train_ration, random_state=random_state)



import yaoysTools.log.logutil as yaoys_log
import yaoysTools.database.mysql as yaoys_mysql
import yaoysTools.uniqueid.snowflake as yaoys_snowflakeid
import yaoysTools.data.split_datautil as yaoys_datautil
import yaoysTools.time.timeutil as yaoys_timeutil
import yaoysTools.toolsutil.mathutil as yaoys_tools_util
import yaoysTools.progress.progressUtil as yaoys_progress_util



# Copyright (c) 2012 Georgios Verigakis <verigak@gmail.com>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.

# The github url: https://github.com/verigak/progress.git

from __future__ import division, print_function

from collections import deque
from datetime import timedelta
from math import ceil
from sys import stderr
from yaoysTools.log.logutil import MY_LOG_DEBUG, log_info, log_error, log_warn, \
    log_debug, MY_LOG_INFO, MY_LOG_ERROR, MY_LOG_WARN, get_log_level

try:
    from time import monotonic
except ImportError:
    from time import time as monotonic

__version__ = '1.6'

HIDE_CURSOR = '\x1b[?25l'
SHOW_CURSOR = '\x1b[?25h'

'''
The show time type is : avg:	simple moving average time per item (in seconds)
                        elapsed:	elapsed time in seconds
                        elapsed_td:	elapsed as a timedelta (useful for printing as a string)
                        eta:	avg * remaining
                        eta_td:	eta as a timedelta (useful for printing as a string)
if you want to define the show time type,you must set the need_show_time is True

if you use spinner,you don't use eta and eta_td
'''


class Infinite(object):
    file = stderr
    sma_window = 10  # Simple Moving Average window
    check_tty = True
    hide_cursor = True

    def __init__(self, message='', **kwargs):
        self.index = 0
        self.start_ts = monotonic()
        self.avg = 0
        self._avg_update_ts = self.start_ts
        self._ts = self.start_ts
        self._xput = deque(maxlen=self.sma_window)

        self._max_width = 0
        self._hidden_cursor = False
        self.message = message

        self.is_save_log = kwargs.get('is_save_log', False)
        self.logger = kwargs.get('logger', None)
        self.check_tty = kwargs.get('check_tty', False)
        self.need_show_time = kwargs.get('need_show_time', True)
        self.color = kwargs.get('color', None)
        self.show_time_type = kwargs.get('show_time_type', 'elapsed_td')
        if self.logger is not None:
            self.log_level = get_log_level(my_logger=self.logger)
        else:
            self.log_level = MY_LOG_DEBUG

        if self.file and self.is_tty():
            if self.hide_cursor:
                print(HIDE_CURSOR, end='', file=self.file)
                self._hidden_cursor = True
        self.writeln('')

    def __del__(self):
        if self._hidden_cursor:
            print(SHOW_CURSOR, end='', file=self.file)

    def __getitem__(self, key):
        if key.startswith('_'):
            return None
        return getattr(self, key, None)

    @property
    def elapsed(self):
        return int(monotonic() - self.start_ts)

    @property
    def elapsed_td(self):
        return timedelta(seconds=self.elapsed)

    def update_avg(self, n, dt):
        if n > 0:
            xput_len = len(self._xput)
            self._xput.append(dt / n)
            now = monotonic()
            # update when we're still filling _xput, then after every second
            if (xput_len < self.sma_window or
                    now - self._avg_update_ts > 1):
                self.avg = sum(self._xput) / len(self._xput)
                self._avg_update_ts = now

    def update(self):
        pass

    def start(self):
        pass

    def writeln(self, line):
        if self.file and self.is_tty():
            width = len(line)
            if width < self._max_width:
                # Add padding to cover previous contents
                line += ' ' * (self._max_width - width)
            else:
                self._max_width = width
            print('\r' + line, end='', file=self.file)
            self.file.flush()

            if self.is_save_log is True:
                if self.log_level == MY_LOG_DEBUG:
                    log_debug(line, my_logger=self.logger)
                if self.log_level == MY_LOG_INFO:
                    log_info(line, my_logger=self.logger)
                if self.log_level == MY_LOG_ERROR:
                    log_error(line, my_logger=self.logger)
                if self.log_level == MY_LOG_WARN:
                    log_warn(line, my_logger=self.logger)

    def finish(self):
        if self.file and self.is_tty():
            print(file=self.file)
            if self._hidden_cursor:
                print(SHOW_CURSOR, end='', file=self.file)
                self._hidden_cursor = False

    def is_tty(self):
        try:
            return self.file.isatty() if self.check_tty else True
        except AttributeError:
            msg = "%s has no attribute 'isatty'. Try setting check_tty=False." % self
            raise AttributeError(msg)

    def next(self, n=1):
        now = monotonic()
        dt = now - self._ts
        self.update_avg(n, dt)
        self._ts = now
        self.index = self.index + n
        self.update()

    def iter(self, it):
        self.iter_value = None
        with self:
            for x in it:
                self.iter_value = x
                yield x
                self.next()
        del self.iter_value

    def __enter__(self):
        self.start()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.finish()


class Progress(Infinite):
    def __init__(self, *args, **kwargs):
        super(Progress, self).__init__(*args, **kwargs)
        self.max = kwargs.get('max', 100)

    @property
    def eta(self):
        return int(ceil(self.avg * self.remaining))

    @property
    def eta_td(self):
        return timedelta(seconds=self.eta)

    @property
    def percent(self):
        return self.progress * 100

    @property
    def progress(self):
        if self.max == 0:
            return 0
        return min(1, self.index / self.max)

    @property
    def remaining(self):
        return max(self.max - self.index, 0)

    def start(self):
        self.update()

    def goto(self, index):
        incr = index - self.index
        self.next(incr)

    def iter(self, it):
        try:
            self.max = len(it)
        except TypeError:
            pass

        self.iter_value = None
        with self:
            for x in it:
                self.iter_value = x
                yield x
                self.next()
        del self.iter_value
